@@ -1,6 +1,6 @@
 --- Get list of legacy filenames
 return function(ctx)
     return {
-        ".yvmrc"
+        ".yvmrc",
     }
-end
\ No newline at end of file
+end
