@@ -512,7 +512,7 @@ impl HttpBackend {
     ) -> Result<()> {
         let settings = Settings::get();
         let filename = file_path.file_name().unwrap().to_string_lossy();
-        let lockfile_enabled = settings.lockfile;
+        let lockfile_enabled = settings.lockfile_enabled();
 
         let platform_key = self.get_platform_key();
         let platform_info = tv.lock_platforms.entry(platform_key).or_default();
@@ -652,7 +652,7 @@ impl Backend for HttpBackend {
 
         // For lockfile checksum verification
         let settings = Settings::get();
-        let lockfile_enabled = settings.lockfile;
+        let lockfile_enabled = settings.lockfile_enabled();
         let has_lockfile_checksum = tv
             .lock_platforms
             .get(&platform_key)
