@@ -5,13 +5,13 @@ use crate::backend::backend_type::BackendType;
 use crate::cache::{CacheManager, CacheManagerBuilder};
 use crate::cli::args::BackendArg;
 use crate::cmd::CmdLineRunner;
+use crate::config::settings::NpmPackageManager;
 use crate::config::{Config, Settings};
 use crate::install_context::InstallContext;
 use crate::timeout;
 use crate::toolset::ToolVersion;
 use async_trait::async_trait;
 use serde_json::Value;
-use std::collections::HashMap;
 use std::{fmt::Debug, sync::Arc};
 use tokio::sync::Mutex as TokioMutex;
 
@@ -39,22 +39,22 @@ impl Backend for NPMBackend {
         // package manager for installation. We avoid listing all package managers to
         // prevent incorrect dependency edges.
         let settings = Settings::get();
-        let package_manager = settings.npm.package_manager.as_str();
+        let package_manager = settings.npm.package_manager;
         let tool_name = self.tool_name();
 
         // Avoid circular dependency when installing npm itself
         // But we still need the configured package manager for installation
         if tool_name == "npm" {
             return match package_manager {
-                "bun" => Ok(vec!["node", "bun"]),
-                "pnpm" => Ok(vec!["node", "pnpm"]),
-                _ => Ok(vec!["node"]),
+                NpmPackageManager::Bun => Ok(vec!["node", "bun"]),
+                NpmPackageManager::Pnpm => Ok(vec!["node", "pnpm"]),
+                NpmPackageManager::Npm => Ok(vec!["node"]),
             };
         }
 
         // Avoid circular dependency when installing the configured package manager
         // e.g., npm:bun with bun configured, or npm:pnpm with pnpm configured
-        if tool_name == package_manager {
+        if tool_name == package_manager.to_string() {
             // Still need npm for version queries
             return Ok(vec!["node", "npm"]);
         }
@@ -62,10 +62,9 @@ impl Backend for NPMBackend {
         // For regular packages: need npm (for version queries) + configured package manager
         let mut deps = vec!["node", "npm"];
         match package_manager {
-            "bun" => deps.push("bun"),
-            "pnpm" => deps.push("pnpm"),
-            // npm is already in deps
-            _ => {}
+            NpmPackageManager::Bun => deps.push("bun"),
+            NpmPackageManager::Pnpm => deps.push("pnpm"),
+            NpmPackageManager::Npm => {} // npm is already in deps
         }
         Ok(deps)
     }
@@ -83,26 +82,34 @@ impl Backend for NPMBackend {
             async || {
                 let env = self.dependency_env(config).await?;
 
-                // Fetch versions and timestamps in parallel
-                let versions_raw =
-                    cmd!(NPM_PROGRAM, "view", self.tool_name(), "versions", "--json")
-                        .full_env(&env)
-                        .env("NPM_CONFIG_UPDATE_NOTIFIER", "false")
-                        .read()?;
-                let time_raw = cmd!(NPM_PROGRAM, "view", self.tool_name(), "time", "--json")
-                    .full_env(&env)
-                    .env("NPM_CONFIG_UPDATE_NOTIFIER", "false")
-                    .read()?;
-
-                let versions: Vec<String> = serde_json::from_str(&versions_raw)?;
-                let time: HashMap<String, String> = serde_json::from_str(&time_raw)?;
-
+                let raw = cmd!(
+                    NPM_PROGRAM,
+                    "view",
+                    self.tool_name(),
+                    "versions",
+                    "time",
+                    "--json"
+                )
+                .full_env(&env)
+                .env("NPM_CONFIG_UPDATE_NOTIFIER", "false")
+                .read()?;
+                let data: Value = serde_json::from_str(&raw)?;
+                let versions = data["versions"]
+                    .as_array()
+                    .ok_or_else(|| eyre::eyre!("invalid versions"))?;
+                let time = data["time"]
+                    .as_object()
+                    .ok_or_else(|| eyre::eyre!("invalid time"))?;
                 let version_info = versions
-                    .into_iter()
+                    .iter()
+                    .filter_map(|v| v.as_str())
                     .map(|version| {
-                        let created_at = time.get(&version).cloned();
+                        let created_at = time
+                            .get(version)
+                            .and_then(|v| v.as_str())
+                            .map(|s| s.to_string());
                         VersionInfo {
-                            version,
+                            version: version.to_string(),
                             created_at,
                             ..Default::default()
                         }
@@ -149,8 +156,8 @@ impl Backend for NPMBackend {
 
     async fn install_version_(&self, ctx: &InstallContext, tv: ToolVersion) -> Result<ToolVersion> {
         self.check_install_deps(&ctx.config).await;
-        match Settings::get().npm.package_manager.as_str() {
-            "bun" => {
+        match Settings::get().npm.package_manager {
+            NpmPackageManager::Bun => {
                 CmdLineRunner::new("bun")
                     .arg("install")
                     .arg(format!("{}@{}", self.tool_name(), tv.version))
@@ -174,7 +181,7 @@ impl Backend for NPMBackend {
                     .current_dir(tv.install_path())
                     .execute()?;
             }
-            "pnpm" => {
+            NpmPackageManager::Pnpm => {
                 let bin_dir = tv.install_path().join("bin");
                 crate::file::create_dir_all(&bin_dir)?;
                 CmdLineRunner::new("pnpm")
@@ -208,6 +215,7 @@ impl Backend for NPMBackend {
                     .arg(tv.install_path())
                     .with_pr(ctx.pr.as_ref())
                     .envs(ctx.ts.env_with_path(&ctx.config).await?)
+                    .env("NPM_CONFIG_UPDATE_NOTIFIER", "false")
                     .prepend_path(ctx.ts.list_paths(&ctx.config).await)?
                     .prepend_path(
                         self.dependency_toolset(&ctx.config)
@@ -227,7 +235,7 @@ impl Backend for NPMBackend {
         _config: &Arc<Config>,
         tv: &crate::toolset::ToolVersion,
     ) -> eyre::Result<Vec<std::path::PathBuf>> {
-        if Settings::get().npm.package_manager == "npm" {
+        if Settings::get().npm.package_manager == NpmPackageManager::Npm {
             Ok(vec![tv.install_path()])
         } else {
             Ok(vec![tv.install_path().join("bin")])
@@ -263,8 +271,8 @@ impl NPMBackend {
 
     /// Check dependencies for package installation (npm or bun based on settings)
     async fn check_install_deps(&self, config: &Arc<Config>) {
-        match Settings::get().npm.package_manager.as_str() {
-            "bun" => {
+        match Settings::get().npm.package_manager {
+            NpmPackageManager::Bun => {
                 self.warn_if_dependency_missing(
                     config,
                     "bun",
@@ -275,7 +283,7 @@ impl NPMBackend {
                 )
                 .await
             }
-            "pnpm" => {
+            NpmPackageManager::Pnpm => {
                 self.warn_if_dependency_missing(
                     config,
                     "pnpm",
