@@ -326,7 +326,7 @@ impl Backend for UbiBackend {
             } else {
                 bail!("Invalid checksum: {platform_key}");
             }
-        } else if Settings::get().lockfile {
+        } else if Settings::get().lockfile_enabled() {
             ctx.pr
                 .set_message(format!("checksum generate {platform_key}"));
             let hash = hash::file_hash_blake3(file, Some(ctx.pr.as_ref()))?;
