@@ -126,25 +126,6 @@ impl Exec {
 
         let (program, mut args) = parse_command(&env::SHELL, &self.command, &self.c);
 
-        // On Unix, resolve the program to its full mise-installed path if it is
-        // provided by a mise-managed tool. This prevents infinite recursion when
-        // a wrapper script (e.g., .devcontainer/bin/tool) calls `mise x -- tool`:
-        // without this, execvp would find the wrapper again (since it precedes
-        // the mise install path in PATH) and loop until E2BIG.
-        // On Windows, exec_program uses which::which_in with PATHEXT to resolve
-        // the correct executable (.cmd/.exe), so we skip this to avoid passing
-        // a pre-resolved path that bypasses Windows-specific extension handling.
-        #[cfg(unix)]
-        let program = if !program.contains('/') {
-            if let Some(bin) = ts.which_bin(&config, &program).await {
-                bin.to_string_lossy().to_string()
-            } else {
-                program
-            }
-        } else {
-            program
-        };
-
         let mut env = measure!("env_with_path", { ts.env_with_path(&config).await? });
 
         // Run auto-enabled prepare steps (unless --no-prepare)
