@@ -4,17 +4,16 @@ use std::fmt::{Debug, Display, Formatter};
 use std::io::{BufRead, BufReader, Write};
 use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus, Stdio};
+use std::sync::atomic::{AtomicU8, Ordering};
 use std::sync::mpsc::channel;
-use std::sync::{Arc, Mutex, RwLock};
+use std::sync::{Arc, Condvar, Mutex, MutexGuard, RwLock};
 use std::thread;
-
-#[cfg(unix)]
-use std::os::unix::process::CommandExt;
+use std::time::Duration;
 
 use crate::redactions::Redactor;
 use color_eyre::Result;
 use duct::{Expression, IntoExecutablePath};
-use eyre::Context;
+use eyre::{Context, bail};
 #[cfg(not(any(test, target_os = "windows")))]
 use signal_hook::consts::{SIGHUP, SIGINT, SIGQUIT, SIGTERM, SIGUSR1, SIGUSR2};
 #[cfg(not(any(test, target_os = "windows")))]
@@ -110,6 +109,120 @@ pub struct CmdLineRunner<'a> {
     pass_signals: bool,
     on_stdout: Option<Box<dyn Fn(String) + Send + 'a>>,
     on_stderr: Option<Box<dyn Fn(String) + Send + 'a>>,
+    timeout: Option<Duration>,
+}
+
+const GUARD_RUNNING: u8 = 0;
+const GUARD_CANCELLED: u8 = 1;
+const GUARD_TIMED_OUT: u8 = 2;
+
+fn wait_for_cancel_or_deadline<'a>(
+    cvar: &'a Condvar,
+    mut guard: MutexGuard<'a, bool>,
+    deadline: std::time::Instant,
+) -> (MutexGuard<'a, bool>, bool) {
+    loop {
+        if *guard {
+            return (guard, true);
+        }
+        let remaining = deadline.saturating_duration_since(std::time::Instant::now());
+        if remaining.is_zero() {
+            return (guard, false);
+        }
+        let (g, result) = cvar.wait_timeout(guard, remaining).unwrap();
+        guard = g;
+        if result.timed_out() {
+            return (guard, false);
+        }
+    }
+}
+
+struct TimeoutGuard {
+    state: Arc<AtomicU8>,
+    cancel: Arc<(Mutex<bool>, Condvar)>,
+    timeout: Duration,
+}
+
+impl TimeoutGuard {
+    fn new(timeout: Duration, pid: u32) -> Self {
+        let state = Arc::new(AtomicU8::new(GUARD_RUNNING));
+        let cancel = Arc::new((Mutex::new(false), Condvar::new()));
+        let state_clone = state.clone();
+        let cancel_clone = cancel.clone();
+        thread::spawn(move || {
+            let (lock, cvar) = &*cancel_clone;
+            let guard = lock.lock().unwrap();
+            let deadline = std::time::Instant::now() + timeout;
+            let (guard, cancelled) = wait_for_cancel_or_deadline(cvar, guard, deadline);
+            if cancelled {
+                return;
+            }
+            if state_clone
+                .compare_exchange(
+                    GUARD_RUNNING,
+                    GUARD_TIMED_OUT,
+                    Ordering::AcqRel,
+                    Ordering::Acquire,
+                )
+                .is_err()
+            {
+                return;
+            }
+            #[cfg(unix)]
+            {
+                let pid = nix::unistd::Pid::from_raw(pid as i32);
+                let _ = nix::sys::signal::kill(pid, nix::sys::signal::Signal::SIGTERM);
+                drop(guard);
+                let guard = lock.lock().unwrap();
+                let grace_deadline = std::time::Instant::now() + Duration::from_secs(5);
+                let (_guard, cancelled) = wait_for_cancel_or_deadline(cvar, guard, grace_deadline);
+                if !cancelled {
+                    let _ = nix::sys::signal::kill(pid, nix::sys::signal::Signal::SIGKILL);
+                }
+            }
+            #[cfg(windows)]
+            {
+                drop(guard);
+                // TODO: Windows lacks graceful shutdown parity with Unix.
+                // Currently force-kills immediately via taskkill /F with no grace period.
+                // Consider using GenerateConsoleCtrlEvent for CTRL_C_EVENT before force kill.
+                let _ = Command::new("taskkill")
+                    .args(["/F", "/PID", &pid.to_string()])
+                    .stdout(Stdio::null())
+                    .stderr(Stdio::null())
+                    .status();
+            }
+        });
+        Self {
+            state,
+            cancel,
+            timeout,
+        }
+    }
+
+    fn cancel(&self) {
+        self.state
+            .compare_exchange(
+                GUARD_RUNNING,
+                GUARD_CANCELLED,
+                Ordering::AcqRel,
+                Ordering::Acquire,
+            )
+            .ok();
+        let (lock, cvar) = &*self.cancel;
+        *lock.lock().unwrap() = true;
+        cvar.notify_one();
+    }
+
+    fn timed_out(&self) -> Option<Duration> {
+        (self.state.load(Ordering::Acquire) == GUARD_TIMED_OUT).then_some(self.timeout)
+    }
+}
+
+impl Drop for TimeoutGuard {
+    fn drop(&mut self) {
+        self.cancel();
+    }
 }
 
 static OUTPUT_LOCK: Mutex<()> = Mutex::new(());
@@ -133,6 +246,7 @@ impl<'a> CmdLineRunner<'a> {
             pass_signals: false,
             on_stdout: None,
             on_stderr: None,
+            timeout: None,
         }
     }
 
@@ -141,13 +255,9 @@ impl<'a> CmdLineRunner<'a> {
         let pids = RUNNING_PIDS.lock().unwrap();
         for pid in pids.iter() {
             let pid = *pid as i32;
-            let nix_pid = nix::unistd::Pid::from_raw(pid);
-            trace!("{signal}: pgid {pid}");
-            if nix::sys::signal::killpg(nix_pid, signal).is_err() {
-                trace!("killpg failed for {pid}, falling back to kill");
-                if let Err(e) = nix::sys::signal::kill(nix_pid, signal) {
-                    debug!("Failed to kill cmd {pid}: {e}");
-                }
+            trace!("{signal}: {pid}");
+            if let Err(e) = nix::sys::signal::kill(nix::unistd::Pid::from_raw(pid), signal) {
+                debug!("Failed to kill cmd {pid}: {e}");
             }
         }
     }
@@ -290,6 +400,11 @@ impl<'a> CmdLineRunner<'a> {
         self
     }
 
+    pub fn with_timeout(mut self, timeout: Duration) -> Self {
+        self.timeout = Some(timeout);
+        self
+    }
+
     pub fn stdin_string(mut self, input: impl Into<String>) -> Self {
         self.cmd.stdin(Stdio::piped());
         self.stdin = Some(input.into());
@@ -306,25 +421,6 @@ impl<'a> CmdLineRunner<'a> {
             let _write_lock = RAW_LOCK.write().unwrap();
             return self.execute_raw();
         }
-        #[cfg(unix)]
-        unsafe {
-            self.cmd.pre_exec(|| {
-                // Don't create a new process group when stdin is a TTY.
-                // Interactive tools (e.g. Tilt) need to stay in the terminal's
-                // foreground process group; moving them to a new one triggers
-                // SIGTTIN and hangs them.
-                // Use BorrowedFd::borrow_raw rather than std::io::stdin() —
-                // pre_exec runs post-fork where OnceLock/malloc are not safe.
-                let stdin = std::os::fd::BorrowedFd::borrow_raw(0);
-                if !std::io::IsTerminal::is_terminal(&stdin) {
-                    let _ = nix::unistd::setpgid(
-                        nix::unistd::Pid::from_raw(0),
-                        nix::unistd::Pid::from_raw(0),
-                    );
-                }
-                Ok(())
-            });
-        }
         let mut cp = self
             .spawn_with_etxtbsy_retry()
             .wrap_err_with(|| format!("failed to execute command: {self}"))?;
@@ -339,7 +435,9 @@ impl<'a> CmdLineRunner<'a> {
                 move || {
                     for line in BufReader::new(stdout).lines() {
                         match line {
-                            Ok(line) => tx.send(ChildProcessOutput::Stdout(line)).unwrap(),
+                            Ok(line) => {
+                                let _ = tx.send(ChildProcessOutput::Stdout(line));
+                            }
                             Err(e) => warn!("Failed to read stdout for {name}: {e}"),
                         }
                     }
@@ -353,7 +451,9 @@ impl<'a> CmdLineRunner<'a> {
                 move || {
                     for line in BufReader::new(stderr).lines() {
                         match line {
-                            Ok(line) => tx.send(ChildProcessOutput::Stderr(line)).unwrap(),
+                            Ok(line) => {
+                                let _ = tx.send(ChildProcessOutput::Stderr(line));
+                            }
                             Err(e) => warn!("Failed to read stderr for {name}: {e}"),
                         }
                     }
@@ -376,7 +476,7 @@ impl<'a> CmdLineRunner<'a> {
             let tx = tx.clone();
             thread::spawn(move || {
                 for sig in &mut signals {
-                    tx.send(ChildProcessOutput::Signal(sig)).unwrap();
+                    let _ = tx.send(ChildProcessOutput::Signal(sig));
                 }
             });
         }
@@ -386,9 +486,11 @@ impl<'a> CmdLineRunner<'a> {
             if let Some(sighandle) = sighandle {
                 sighandle.close();
             }
-            tx.send(ChildProcessOutput::ExitStatus(status)).unwrap();
+            let _ = tx.send(ChildProcessOutput::ExitStatus(status));
         });
 
+        let timeout_guard = self.timeout.map(|t| TimeoutGuard::new(t, id));
+
         let mut combined_output = vec![];
         let mut status = None;
         for line in rx {
@@ -404,24 +506,34 @@ impl<'a> CmdLineRunner<'a> {
                     combined_output.push((line, OutputSource::Stderr));
                 }
                 ChildProcessOutput::ExitStatus(s) => {
-                    RUNNING_PIDS.lock().unwrap().remove(&id);
                     status = Some(s);
                 }
                 #[cfg(not(any(test, windows)))]
                 ChildProcessOutput::Signal(sig) => {
-                    debug!("Received signal {sig}, forwarding to pgid {id}");
-                    let pgid = nix::unistd::Pid::from_raw(id as i32);
-                    let sig = nix::sys::signal::Signal::try_from(sig).unwrap();
-                    if nix::sys::signal::killpg(pgid, sig).is_err() {
-                        let _ = nix::sys::signal::kill(pgid, sig);
+                    if sig != SIGINT {
+                        debug!("Received signal {sig}, forwarding to {id}");
+                        let pid = nix::unistd::Pid::from_raw(id as i32);
+                        let sig = nix::sys::signal::Signal::try_from(sig).unwrap();
+                        // Ignore errors — the child may have already exited
+                        // (e.g., killed by TimeoutGuard or exited naturally).
+                        let _ = nix::sys::signal::kill(pid, sig);
                     }
                 }
             }
         }
+        // Removed after rx loop drains (not inside ExitStatus arm) so kill_all
+        // can still reach this PID while output is being processed.
         RUNNING_PIDS.lock().unwrap().remove(&id);
+        if let Some(g) = &timeout_guard {
+            g.cancel();
+        }
+
         let status = status.unwrap();
 
         if !status.success() {
+            if let Some(duration) = timeout_guard.as_ref().and_then(|g| g.timed_out()) {
+                bail!("timed out after {duration:?}");
+            }
             self.on_error(combined_output, status)?;
         }
 
@@ -429,11 +541,19 @@ impl<'a> CmdLineRunner<'a> {
     }
 
     fn execute_raw(mut self) -> Result<()> {
-        let status = self.spawn_with_etxtbsy_retry()?.wait()?;
-        match status.success() {
-            true => Ok(()),
-            false => self.on_error(vec![], status),
+        let mut cp = self.spawn_with_etxtbsy_retry()?;
+        let timeout_guard = self.timeout.map(|t| TimeoutGuard::new(t, cp.id()));
+        let status = cp.wait()?;
+        if let Some(g) = &timeout_guard {
+            g.cancel();
         }
+        if !status.success() {
+            if let Some(duration) = timeout_guard.as_ref().and_then(|g| g.timed_out()) {
+                bail!("timed out after {duration:?}");
+            }
+            return self.on_error(vec![], status);
+        }
+        Ok(())
     }
 
     /// Retry spawning a process if it fails with ETXTBSY (Text file busy).
