@@ -394,6 +394,47 @@ impl Lockfile {
         platforms
     }
 
+    /// Keep only tools matching configured short names or backend identifiers.
+    /// Also removes conda packages that become unreferenced.
+    pub fn retain_tools_by_short_or_backend(
+        &mut self,
+        keep_shorts: &BTreeSet<String>,
+        keep_backends: &BTreeSet<String>,
+    ) {
+        self.tools.retain(|short, versions| {
+            Self::should_keep_tool(short, versions, keep_shorts, keep_backends)
+        });
+        self.cleanup_unreferenced_conda_packages();
+    }
+
+    /// Return tool keys that would be removed by `retain_tools_by_short_or_backend`.
+    pub fn stale_tool_shorts(
+        &self,
+        keep_shorts: &BTreeSet<String>,
+        keep_backends: &BTreeSet<String>,
+    ) -> BTreeSet<String> {
+        self.tools
+            .iter()
+            .filter_map(|(short, versions)| {
+                (!Self::should_keep_tool(short, versions, keep_shorts, keep_backends))
+                    .then_some(short.clone())
+            })
+            .collect()
+    }
+
+    fn should_keep_tool(
+        short: &str,
+        versions: &[LockfileTool],
+        keep_shorts: &BTreeSet<String>,
+        keep_backends: &BTreeSet<String>,
+    ) -> bool {
+        keep_shorts.contains(short)
+            || versions
+                .iter()
+                .filter_map(|v| v.backend.as_ref())
+                .any(|backend| keep_backends.contains(backend))
+    }
+
     /// Update or add platform info for a tool version
     /// Merges with existing info, preserving fields we don't have new values for
     pub fn set_platform_info(
@@ -516,7 +557,7 @@ fn extract_env_from_config_path(path: &Path) -> Option<String> {
 }
 
 pub fn update_lockfiles(config: &Config, ts: &Toolset, new_versions: &[ToolVersion]) -> Result<()> {
-    if !Settings::get().lockfile {
+    if !Settings::get().lockfile_enabled() || Settings::get().locked {
         return Ok(());
     }
 
@@ -677,7 +718,7 @@ fn determine_target_platforms_from_lockfile(lockfile: Option<&Lockfile>) -> Vec<
 /// so the lockfile is complete and doesn't change when other developers on different
 /// platforms run `mise install`.
 pub async fn auto_lock_new_versions(_config: &Config, new_versions: &[ToolVersion]) -> Result<()> {
-    if !Settings::get().lockfile || new_versions.is_empty() {
+    if !Settings::get().lockfile_enabled() || Settings::get().locked || new_versions.is_empty() {
         return Ok(());
     }
 
@@ -1066,7 +1107,8 @@ pub fn get_locked_version(
     prefix: &str,
     request_options: &BTreeMap<String, String>,
 ) -> Result<Option<LockfileTool>> {
-    if !Settings::get().lockfile {
+    let settings = Settings::get();
+    if !settings.lockfile_enabled() {
         return Ok(None);
     }
 
@@ -1145,7 +1187,8 @@ pub fn get_locked_version(
 /// Get the backend for a tool from the lockfile, ignoring options.
 /// This is used for backend discovery where we just need any entry's backend.
 pub fn get_locked_backend(config: &Config, short: &str) -> Option<String> {
-    if !Settings::get().lockfile {
+    let settings = Settings::get();
+    if !settings.lockfile_enabled() {
         return None;
     }
 
@@ -1364,7 +1407,54 @@ fn format(mut doc: DocumentMut) -> String {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use std::collections::BTreeMap;
+    use std::collections::{BTreeMap, BTreeSet};
+
+    fn basic_tool(version: &str, backend: &str) -> LockfileTool {
+        LockfileTool {
+            version: version.to_string(),
+            backend: Some(backend.to_string()),
+            options: BTreeMap::new(),
+            env: None,
+            platforms: BTreeMap::new(),
+        }
+    }
+
+    fn tool_with_conda_dep(
+        version: &str,
+        backend: &str,
+        platform: &str,
+        dep: &str,
+    ) -> LockfileTool {
+        let mut platforms = BTreeMap::new();
+        platforms.insert(
+            platform.to_string(),
+            PlatformInfo {
+                checksum: None,
+                size: None,
+                url: None,
+                url_api: None,
+                conda_deps: Some(vec![dep.to_string()]),
+            },
+        );
+        LockfileTool {
+            version: version.to_string(),
+            backend: Some(backend.to_string()),
+            options: BTreeMap::new(),
+            env: None,
+            platforms,
+        }
+    }
+
+    fn add_test_conda_package(lockfile: &mut Lockfile, platform: &str, basename: &str) {
+        lockfile.set_conda_package(
+            platform,
+            basename,
+            CondaPackageInfo {
+                url: format!("https://example.com/{basename}.conda"),
+                checksum: Some(format!("sha256:{basename}")),
+            },
+        );
+    }
 
     #[test]
     fn test_array_format_required() {
@@ -1804,6 +1894,109 @@ backend = "conda:jq"
         assert!(!lockfile.conda_packages["macos-arm64"].contains_key("unreferenced-pkg"));
     }
 
+    #[test]
+    fn test_retain_tools_by_short_prunes_removed_tools() {
+        let mut lockfile = Lockfile::default();
+        lockfile
+            .tools
+            .insert("dummy".to_string(), vec![basic_tool("1.0.0", "asdf:dummy")]);
+        lockfile
+            .tools
+            .insert("tiny".to_string(), vec![basic_tool("2.1.0", "asdf:tiny")]);
+
+        let keep_shorts = BTreeSet::from(["tiny".to_string()]);
+        lockfile.retain_tools_by_short_or_backend(&keep_shorts, &BTreeSet::new());
+
+        assert!(!lockfile.tools.contains_key("dummy"));
+        assert!(lockfile.tools.contains_key("tiny"));
+    }
+
+    #[test]
+    fn test_stale_tool_shorts_identifies_removed_tools() {
+        let mut lockfile = Lockfile::default();
+        lockfile
+            .tools
+            .insert("dummy".to_string(), vec![basic_tool("1.0.0", "asdf:dummy")]);
+        lockfile
+            .tools
+            .insert("tiny".to_string(), vec![basic_tool("2.1.0", "asdf:tiny")]);
+
+        let keep_shorts = BTreeSet::from(["tiny".to_string()]);
+        let stale = lockfile.stale_tool_shorts(&keep_shorts, &BTreeSet::new());
+        assert_eq!(stale, BTreeSet::from(["dummy".to_string()]));
+    }
+
+    #[test]
+    fn test_stale_tool_shorts_respects_backend_identifiers() {
+        let mut lockfile = Lockfile::default();
+        lockfile.tools.insert(
+            "jq".to_string(),
+            vec![basic_tool("1.7.1", "aqua:jqlang/jq")],
+        );
+
+        let keep_backends = BTreeSet::from(["aqua:jqlang/jq".to_string()]);
+        let stale = lockfile.stale_tool_shorts(&BTreeSet::new(), &keep_backends);
+        assert!(stale.is_empty());
+    }
+
+    #[test]
+    fn test_retain_tools_by_short_cleans_unreferenced_conda_packages() {
+        let mut lockfile = Lockfile::default();
+
+        add_test_conda_package(&mut lockfile, "linux-x64", "keep-pkg");
+        add_test_conda_package(&mut lockfile, "linux-x64", "drop-pkg");
+
+        lockfile.tools.insert(
+            "tiny".to_string(),
+            vec![tool_with_conda_dep(
+                "2.1.0",
+                "conda:tiny",
+                "linux-x64",
+                "keep-pkg",
+            )],
+        );
+        lockfile.tools.insert(
+            "dummy".to_string(),
+            vec![tool_with_conda_dep(
+                "1.0.0",
+                "conda:dummy",
+                "linux-x64",
+                "drop-pkg",
+            )],
+        );
+
+        let keep_shorts = BTreeSet::from(["tiny".to_string()]);
+        lockfile.retain_tools_by_short_or_backend(&keep_shorts, &BTreeSet::new());
+
+        assert!(lockfile.tools.contains_key("tiny"));
+        assert!(!lockfile.tools.contains_key("dummy"));
+
+        let linux_packages = lockfile.conda_packages.get("linux-x64").unwrap();
+        assert!(linux_packages.contains_key("keep-pkg"));
+        assert!(!linux_packages.contains_key("drop-pkg"));
+    }
+
+    #[test]
+    fn test_retain_tools_by_short_or_backend_preserves_legacy_keyed_entries() {
+        let mut lockfile = Lockfile::default();
+        lockfile.tools.insert(
+            "jq".to_string(),
+            vec![LockfileTool {
+                version: "1.7.1".to_string(),
+                backend: Some("aqua:jqlang/jq".to_string()),
+                options: BTreeMap::new(),
+                env: None,
+                platforms: BTreeMap::new(),
+            }],
+        );
+
+        let keep_shorts = BTreeSet::from(["aqua:jqlang/jq".to_string()]);
+        let keep_backends = BTreeSet::from(["aqua:jqlang/jq".to_string()]);
+        lockfile.retain_tools_by_short_or_backend(&keep_shorts, &keep_backends);
+
+        assert!(lockfile.tools.contains_key("jq"));
+    }
+
     #[test]
     fn test_platform_info_merge_prefers_sha256() {
         // Test that merge_with prefers sha256 over blake3
